package sun.awt;

import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.util.Vector;
import java.util.Locale;
import sun.java2d.FontSupport;
import sun.font.SunFontManager;
import java.awt.peer.FontPeer;

public abstract class PlatformFont implements FontPeer
{
    protected FontDescriptor[] componentFonts;
    protected char defaultChar;
    protected FontConfiguration fontConfig;
    protected FontDescriptor defaultFont;
    protected String familyName;
    private Object[] fontCache;
    protected static int FONTCACHESIZE;
    protected static int FONTCACHEMASK;
    protected static String osVersion;
    
    public PlatformFont(String s, int n) {
        SunFontManager instance = SunFontManager.getInstance();
        if ((instance instanceof FontSupport)) {
            this.fontConfig = instance.getFontConfiguration();
        }
        if (this.fontConfig == null) {
            return;
        }
        this.familyName = s.toLowerCase(Locale.ENGLISH);
        if (!FontConfiguration.isLogicalFontFamilyName(this.familyName)) {
            this.familyName = this.fontConfig.getFallbackFamilyName(this.familyName, "sansserif");
        }
        this.componentFonts = this.fontConfig.getFontDescriptors(this.familyName, n);
        int missingGlyphCharacter = this.getMissingGlyphCharacter();
        this.defaultChar = '?';
        if (this.componentFonts.length > 0) {
            this.defaultFont = this.componentFonts[0];
        }
        for (int i = 0; i < this.componentFonts.length; ++i) {
            if (!this.componentFonts[i].isExcluded((char)missingGlyphCharacter) && this.componentFonts[i].encoder.canEncode((char)missingGlyphCharacter)) {
                this.defaultFont = this.componentFonts[i];
                this.defaultChar = missingGlyphCharacter;
                break;
            }
        }
        return;
    }
    
    protected abstract char getMissingGlyphCharacter();
    
    public CharsetString[] makeMultiCharsetString(String s) {
        return this.makeMultiCharsetString(s.toCharArray(), 0, s.length(), true);
    }
    
    public CharsetString[] makeMultiCharsetString(String s, boolean b) {
        return this.makeMultiCharsetString(s.toCharArray(), 0, s.length(), b);
    }
    
    public CharsetString[] makeMultiCharsetString(char[] array, int n, int n2) {
        return this.makeMultiCharsetString(array, n, n2, true);
    }
    
    public CharsetString[] makeMultiCharsetString(char[] array, int n, int n2, boolean b) {
        if (n2 < 1) {
            return new CharsetString[0];
        }
        Object o = null;
        char[] array2 = new char[n2];
        int defaultChar = this.defaultChar;
        boolean n3 = false;
        FontDescriptor defaultFont = this.defaultFont;
        for (int i = 0; i < this.componentFonts.length; ++i) {
            if (!this.componentFonts[i].isExcluded(array[n]) && this.componentFonts[i].encoder.canEncode(array[n])) {
                defaultFont = this.componentFonts[i];
                defaultChar = array[n];
                n3 = true;
                break;
            }
        }
        if (!b && n3 == 0) {
            return null;
        }
        array2[0] = defaultChar;
        int n4 = 0;
        int j = 1;
        while (j < n2) {
            int c = array[n + j];
            FontDescriptor defaultFont2 = this.defaultFont;
            int defaultChar2 = this.defaultChar;
            boolean n5 = false;
            for (int k = 0; k < this.componentFonts.length; ++k) {
                if (!this.componentFonts[k].isExcluded((char)c) && this.componentFonts[k].encoder.canEncode((char)c)) {
                    defaultFont2 = this.componentFonts[k];
                    defaultChar2 = c;
                    n5 = true;
                    break;
                }
            }
            if (!b && n5 == 0) {
                return null;
            }
            array2[j] = defaultChar2;
            if (defaultFont != defaultFont2) {
                if (o == null) {
                    o = new Vector(3);
                }
                o.addElement(new CharsetString(array2, n4, j - n4, defaultFont));
                defaultFont = defaultFont2;
                FontDescriptor defaultFont3 = this.defaultFont;
                n4 = j;
            }
            ++j;
        }
        CharsetString obj = new CharsetString(array2, n4, n2 - n4, defaultFont);
        if (o == null) {
            return new CharsetString[] { obj };
        }
        o.addElement(obj);
        CharsetString[] array3 = new CharsetString[o.size()];
        for (int l = 0; l < o.size(); ++l) {
            array3[l] = (CharsetString)o.elementAt(l);
        }
        return array3;
    }
    
    public boolean mightHaveMultiFontMetrics() {
        return this.fontConfig != null;
    }
    
    public Object[] makeConvertedMultiFontString(String s) {
        return this.makeConvertedMultiFontChars(s.toCharArray(), 0, s.length());
    }
    
    public Object[] makeConvertedMultiFontChars(char[] array, int i, int n) {
        Object[] array2 = new Object[2];
        byte[] array3 = null;
        int n2 = 0;
        int n3 = 0;
        FontDescriptor fontDescriptor = null;
        int n4 = i + n;
        if (i < 0 || n4 > array.length) {
            throw new ArrayIndexOutOfBoundsException();
        }
        if (i >= n4) {
            return null;
        }
        while (i < n4) {
            int n5 = array[i];
            int n6 = n5 & PlatformFont.FONTCACHEMASK;
            PlatformFont.PlatformFontCache platformFontCache = (Object)this.getFontCache()[n6];
            if (platformFontCache == null || platformFontCache.uniChar != n5) {
                FontDescriptor defaultFont = this.defaultFont;
                int defaultChar = this.defaultChar;
                int c = array[i];
                for (int length = this.componentFonts.length, j = 0; j < length; ++j) {
                    FontDescriptor fontDescriptor2 = this.componentFonts[j];
                    fontDescriptor2.encoder.reset();
                    if (!fontDescriptor2.isExcluded((char)c) && fontDescriptor2.encoder.canEncode((char)c)) {
                        defaultFont = fontDescriptor2;
                        defaultChar = c;
                        break;
                    }
                }
                try {
                    char[] array4 = { defaultChar };
                    platformFontCache = new PlatformFont.PlatformFontCache();
                    if (defaultFont.useUnicode()) {
                        if (FontDescriptor.isLE) {
                            platformFontCache.bb.put((byte)(array4[0] & 'ÿ'));
                            platformFontCache.bb.put((byte)(array4[0] >> 8));
                        }
                        else {
                            platformFontCache.bb.put((byte)(array4[0] >> 8));
                            platformFontCache.bb.put((byte)(array4[0] & 'ÿ'));
                        }
                    }
                    else {
                        defaultFont.encoder.encode(CharBuffer.wrap(array4), platformFontCache.bb, true);
                    }
                    platformFontCache.fontDescriptor = defaultFont;
                    platformFontCache.uniChar = array[i];
                    this.getFontCache()[n6] = platformFontCache;
                }
                catch (Exception x) {
                    System.err.println(x);
                    x.printStackTrace();
                    return null;
                }
            }
            if (fontDescriptor != platformFontCache.fontDescriptor) {
                if (fontDescriptor != null) {
                    array2[n3++] = fontDescriptor;
                    if ((array2[n3++] = array3) != null) {
                        n2 -= 4;
                        array3[0] = (byte)(n2 >> 24);
                        array3[1] = (byte)(n2 >> 16);
                        array3[2] = (byte)(n2 >> 8);
                        array3[3] = (byte)n2;
                    }
                    if (n3 >= array2.length) {
                        Object[] array5 = new Object[array2.length * 2];
                        System.arraycopy(array2, 0, array5, 0, array2.length);
                        array2 = array5;
                    }
                }
                if (platformFontCache.fontDescriptor.useUnicode()) {
                    array3 = new byte[(n4 - i + 1) * (int)platformFontCache.fontDescriptor.unicodeEncoder.maxBytesPerChar() + 4];
                }
                else {
                    array3 = new byte[(n4 - i + 1) * (int)platformFontCache.fontDescriptor.encoder.maxBytesPerChar() + 4];
                }
                n2 = 4;
                fontDescriptor = platformFontCache.fontDescriptor;
            }
            byte[] array6 = platformFontCache.bb.array();
            int position = platformFontCache.bb.position();
            if (position == 1) {
                array3[n2++] = array6[0];
            }
            else if (position == 2) {
                array3[n2++] = array6[0];
                array3[n2++] = array6[1];
            }
            else if (position == 3) {
                array3[n2++] = array6[0];
                array3[n2++] = array6[1];
                array3[n2++] = array6[2];
            }
            else if (position == 4) {
                array3[n2++] = array6[0];
                array3[n2++] = array6[1];
                array3[n2++] = array6[2];
                array3[n2++] = array6[3];
            }
            ++i;
        }
        array2[n3++] = fontDescriptor;
        if ((array2[n3] = array3) != null) {
            n2 -= 4;
            array3[0] = (byte)(n2 >> 24);
            array3[1] = (byte)(n2 >> 16);
            array3[2] = (byte)(n2 >> 8);
            array3[3] = (byte)n2;
            return array2;
        }
        return array2;
    }
    
    protected final Object[] getFontCache() {
        if (this.fontCache == null) {
            this.fontCache = new Object[PlatformFont.FONTCACHESIZE];
        }
        return this.fontCache;
    }
    
    private static native void initIDs();
    
    static {
        NativeLibLoader.loadLibraries();
        initIDs();
        PlatformFont.FONTCACHESIZE = 256;
        PlatformFont.FONTCACHEMASK = PlatformFont.FONTCACHESIZE - 1;
    }
    
    class PlatformFontCache
    {
        char uniChar;
        FontDescriptor fontDescriptor;
        ByteBuffer bb;
        
        PlatformFontCache() {
            this.bb = ByteBuffer.allocate(4);
        }
    }
}
